@@ -350,7 +350,7 @@ class KeyboardEvent(InputEvent):
         if self.keyType:
             return self.keyType in KeyboardEvent.TYPE_LOCKING
 
-        lockingKeys = ["Caps_Lock", "Num_Lock", "Scroll_Lock"]
+        lockingKeys = ["Caps_Lock", "Shift_Lock", "Num_Lock", "Scroll_Lock"]
         if not self.event_string in lockingKeys:
             return False
 
@@ -475,6 +475,8 @@ class KeyboardEvent(InputEvent):
 
         if self.event_string == "Caps_Lock":
             mod = pyatspi.MODIFIER_SHIFTLOCK
+        elif self.event_string == "Shift_Lock":
+            mod = pyatspi.MODIFIER_SHIFT
         elif self.event_string == "Num_Lock":
             mod = pyatspi.MODIFIER_NUMLOCK
         else:
@@ -654,6 +656,7 @@ class KeyboardEvent(InputEvent):
 
         if self._bypassOrca:
             if self.event_string == "Caps_Lock" \
+               or self.event_string == "Shift_Lock" \
                and self.type == pyatspi.KEY_PRESSED_EVENT:
                     self._lock_mod()
                     self.keyType = KeyboardEvent.TYPE_LOCKING
@@ -699,10 +702,9 @@ class KeyboardEvent(InputEvent):
         return False, 'Unaddressed case'
 
     def _lock_mod(self):
-        def lock_mod(modifiers):
+        def lock_mod(modifiers, modifier):
             def lockit():
                 try:
-                    modifier = (1 << pyatspi.MODIFIER_SHIFTLOCK)
                     if modifiers & modifier:
                         lock = pyatspi.KEY_UNLOCKMODIFIERS
                         debug.println(debug.LEVEL_INFO, "Locking capslock", True)
@@ -716,8 +718,16 @@ class KeyboardEvent(InputEvent):
                         "at-spi2-core >= 2.32 is needed for triggering capslock", True)
                     pass
             return lockit
+        if self.event_string == "Caps_Lock":
+            modifier = 1 << pyatspi.MODIFIER_SHIFTLOCK
+        elif self.event_string == "Shift_Lock":
+            modifier = 1 << pyatspi.MODIFIER_SHIFT
+        else:
+            msg = "Unknown locking key %s" % self.event_string
+            debug.println(debug.LEVEL_WARNING, msg, True)
+            return
         debug.println(debug.LEVEL_INFO, "Scheduling capslock", True)
-        GLib.timeout_add(1, lock_mod(self.modifiers))
+        GLib.timeout_add(1, lock_mod(self.modifiers, modifier))
 
     def _consume(self):
         startTime = time.time()
@@ -774,7 +784,7 @@ class MouseButtonEvent(InputEvent):
         self.pressed = event.type.endswith('p')
         self.button = event.type[len("mouse:button:"):-1]
         self._script = orca_state.activeScript
-        self.window = None
+        self.window = orca_state.activeWindow
         self.obj = None
 
         if self.pressed:
@@ -783,7 +793,9 @@ class MouseButtonEvent(InputEvent):
         if not self._script:
             return
 
-        self.window = self._script.utilities.activeWindow()
+        if not self._script.utilities.canBeActiveWindow(self.window):
+            self.window = self._script.utilities.activeWindow()
+
         if not self.window:
             return
 
