@@ -98,6 +98,7 @@ class Script(web.Script):
                               pyatspi.ROLE_LIST_BOX,
                               pyatspi.ROLE_MENU,
                               pyatspi.ROLE_REDUNDANT_OBJECT,
+                              pyatspi.ROLE_SECTION,
                               pyatspi.ROLE_TABLE,
                               pyatspi.ROLE_TREE,
                               pyatspi.ROLE_TREE_TABLE,
@@ -163,7 +164,7 @@ class Script(web.Script):
     def onCaretMoved(self, event):
         """Callback for object:text-caret-moved accessibility events."""
 
-        if self.utilities.isStaticTextLeaf(event.source, False):
+        if self.utilities.isStaticTextLeaf(event.source):
             msg = "CHROMIUM: Ignoring event from static-text leaf"
             debug.println(debug.LEVEL_INFO, msg, True)
             return
@@ -193,7 +194,7 @@ class Script(web.Script):
     def onChildrenChanged(self, event):
         """Callback for object:children-changed accessibility events."""
 
-        if self.utilities.isStaticTextLeaf(event.any_data, False):
+        if self.utilities.isStaticTextLeaf(event.any_data):
             msg = "CHROMIUM: Ignoring because child is static text leaf"
             debug.println(debug.LEVEL_INFO, msg, True)
             return
@@ -370,7 +371,7 @@ class Script(web.Script):
     def onTextSelectionChanged(self, event):
         """Callback for object:text-selection-changed accessibility events."""
 
-        if self.utilities.isStaticTextLeaf(event.source, False):
+        if self.utilities.isStaticTextLeaf(event.source):
             msg = "CHROMIUM: Ignoring event from static-text leaf"
             debug.println(debug.LEVEL_INFO, msg, True)
             return
